#pragma once
#include "IOValues/interrupt_io_values.hpp"

namespace JabyEngine {
    using StatusIO = IOPort<Interrupt_IO_Values::Status>;
    using MaskIO   = IOPort<Interrupt_IO_Values::Mask>;

    struct Interrupt {
        static constexpr auto VBlank     = Bit(0);
        static constexpr auto GPU        = Bit(1);
        static constexpr auto CDROM      = Bit(2);
        static constexpr auto DMA        = Bit(3);
        static constexpr auto Timer0     = Bit(4);
        static constexpr auto Timer1     = Bit(5);
        static constexpr auto Timer2     = Bit(6);
        static constexpr auto Periphery  = Bit(7);
        static constexpr auto SIO        = Bit(8);
        static constexpr auto SPU        = Bit(9);
        static constexpr auto Controller = Bit(10);
        static constexpr auto LightPen   = Controller;
        
        static inline auto& Status = __new_declare_io_port(StatusIO, 0x1F801070);
        static inline auto& Mask   = __new_declare_io_port(MaskIO,   0x1F801074);

        static bool is_irq(Bit irq) {
            return Status.read().is_set(irq);
        }

        static void ack_irq(Bit irq) {
            Status.write({bit::clear<uint32_t>(0b11111111111, irq)});
        }

        static void disable_irq(Bit irq) {
            Mask.write(Mask.read().clear(irq));
        }

        static void enable_irq(Bit irq) {
            Mask.write(Mask.read().set(irq));
        }
    };
}